#!/usr/bin/ksh
###################################################################
# Name: 	Korn shell script mqllmaint.ksh 
# Created:      2002/03/15 by Tim von Niessen
# Descr:        Perform MQ Linear Logfile Maintenance
###################################################################
# Describe script usage parameters
USAGE() {
echo "\n"
echo "Usage for program $0 is:"
echo "\n"
echo "$0 -q <QueueManager> [-d <MQDirectory>] [-l <RunLog>] [-a <ArchiveLog>] [-w <WrapDepth>] [-v]"
echo "\n"
echo "Where:"
echo "\t-q <QueueManager>\tSpecifies the name of the Queue Manager"
	echo "\t\t\t\ton which to perform maintenance."
echo "\t-d <MQDirectory>\t[optional] Specify an alternate location for"
	echo "\t\t\t\tthe base MQSeries product, default=/var/mqm"
echo "\t-l <RunLog>\t\t[optional] Specify a specific location for the"
	echo "\t\t\t\toutput from the program run, default=none"
echo "\t-a <ArchiveLog>\t\t[optional] Specify a specific location for the"
	echo "\t\t\t\tarchived logs to be stored, default=current"
	echo "\t\t\t\tQueue Manager logfile directory"
echo "\t-w <WrapDepth>\t\t[optional] Specify the depth to be used for"
	echo "\t\t\t\tdetermining file wrap point, default=10"
echo "\t-x \t\t\t[optional] Utilize an alternate program for the"
	echo "\t\t\t\tMQ Linear Logfile archival process, default=native"
	echo "\t\t\t\tUNIX \"tar\" and \"compress\" utilities."
	echo "\t\t\t\tListing of eligible files will be sent to STDOUT"
	echo "\t\t\t\twhich can be piped to another program for processing."
echo "\t-v \t\t\t[optional] Run program in verbose mode"
}
###################################################################
# Subroutine for error processing
ERR() {
VERBOSITY='Y'
if [ $# -eq 0 ] ; then ERRCODE=86
else ERRCODE=$1 ; fi
SAY "Program $0 failed condition code :: $ERRCODE"
case $ERRCODE in
        1) SAY "\tInvalid input parameters: $PARMS" ; USAGE ;;
        2) SAY "\tInvalid Queue Manager Definition in: \"$PARMS\"" ;;
        3) SAY "\tInvalid Archive Directory Specified: \"$ARCHIVE_DIR\"\n\tDirectory does not exist." ;;
        4) SAY "\tInvalid Logging File Specified: \"$LOG_TO\"\n\tDirectory does not exist." ;;
        5) SAY "\tQueue Manager ini file for \"$QMGR\" does not exist." ;;
        6) SAY "\tMQ Logfiles for \"$QMGR\" are \"$LOGTYPE\" not \"LINEAR\"." ;;
        9) SAY "\tVerbose and external output are mutually exclusive options." ;;
        *) SAY "\tCause UNKNOWN!" ;;
esac
exit $ERRCODE
}
###################################################################
# Redirect output based on run parameters (VERBOSITY & LOGGING)
SAY() {
TOSAY=$*
NOW
if [ "$VERBOSITY" = 'Y' ] ; then
	echo $NOW $*
fi
if [ "$LOGGING" = 'Y' ] ; then
	echo $NOW $* >> $LOG_TO
fi
}
###################################################################
# Set time/date stamp
NOW() { 
NOW=`date +"%Y/%m/%d %T"` 
}
###################################################################
READINLOGS() {
SAY "Reading MQ Error Logs for RECOVERY/RESTART File Definitions."
# Create temporary filtered & sorted files
TMPFILE=$ARCHIVE_DIR/AMQ746.tmp
TMPFILE2=$ARCHIVE_DIR/AMQ746TOO.tmp
FOUND_ENTRIES='N'
for INFILE in `ls -t $QMERRORS/AMQERR0*.LOG` ; do
	if [ $FOUND_ENTRIES = 'Y' ] ; then break
	elif [ `grep -c AMQ7467 $INFILE` -gt 0 -a `grep -c AMQ7467 $INFILE` -gt 0 ] ; then
		SAY "Necessary logfile entries found in $INFILE"
		FOUND_ENTRIES='Y'
	fi
	cat $INFILE | egrep '(AMQ746|\.LOG\.|/)' > $TMPFILE
	cat /dev/null > $TMPFILE2

	#Read in the temporary file
	MSGID=''
	LOGFILE=''
	while read INLINE ; do
		# Find timestamps
		INSTRING \/ $INLINE 
		if [ $? = 0 -a `echo $INLINE | wc -w` -eq 2 ] ; then
			if [ "$MSGID" != '' -a "$LOGFILE" != '' ] ; then
				echo "$TIMESTAMP $MSGID $LOGFILE" >> $TMPFILE2
				MSGID=''
				LOGFILE=''
			fi
			# Reorder timestamp from: mm/dd/yy hh:mm:ss
			TIMESTAMP=`echo $INLINE | tr "/" " "`
			Y=`echo $TIMESTAMP | cut -d" " -f3`
			M=`echo $TIMESTAMP | cut -d" " -f1`
			D=`echo $TIMESTAMP | cut -d" " -f2`
			T=`echo $TIMESTAMP | cut -d" " -f4`
			TIMESTAMP=` echo "$Y $M $D $T" | tr " " "/"`
			continue
		fi
		# Find valid message IDs
		INSTRING AMQ746 $INLINE 
		if [ $? = 0 ] ; then
			MSGID=`echo $INLINE | cut -d":" -f1`
			continue
		fi
		# Find logfile names
		INSTRING LOG $INLINE 
		if [ $? = 0 ] ; then
			TOKENS=`echo $INLINE | wc -w`
			LOGFILE=`echo $INLINE | cut -d" " -f$TOKENS | cut -c2-8`
		fi
	done < $TMPFILE
	# Catch last entry from logfile
	if [ "$MSGID" != '' -a "$LOGFILE" != '' ] ; then
		echo "$TIMESTAMP $MSGID $LOGFILE" >> $TMPFILE2
	fi
done
# Sort output data for second round of processing
cat $TMPFILE2 | sort -r > $TMPFILE
for NEXTMSG in $MQMESSAGES ; do
	while read INLINE ; do
		INSTRING $NEXTMSG $INLINE 
		if [ $? = 0 ] ; then
			SETOLDEST $INLINE
			break
		fi
	done < $TMPFILE
done

# Cleanup temporary files
rm $TMPFILE
rm $TMPFILE2
}
###################################################################
# Check for existence of a value within a string
INSTRING() {
if [ $# -lt 2 ] ; then return 1 ; fi
LOOK4=$1
shift
WITHIN=$*
if [ `echo $WITHIN | grep -c $LOOK4` -eq 0 ] ; then
	return 1
else
	return 0
fi
}
###################################################################
# Set values for OLDEST_RESTART & OLDEST_RECOVERY (with timestamps)
SETOLDEST() {
TIMESTAMP=$1
MSGID=$2
LOGFILE=$3
if [ "$MSGID" = 'AMQ7467' ] ; then
	# Set RESTART
	if [[ "$TIMESTAMP" > "$OLDEST_RESTART_STAMP" ]] ; then
		SAY "Set RESTART to $LOGFILE as of $TIMESTAMP"
		OLDEST_RESTART_STAMP=$TIMESTAMP
		OLDEST_RESTART=$LOGFILE
	fi
elif [ "$MSGID" = 'AMQ7468' ] ; then
	# Set RECOVERY
	if [[ "$TIMESTAMP" > "$OLDEST_RECOVER_STAMP" ]] ; then
		SAY "Set RECOVERY to $LOGFILE as of $TIMESTAMP"
		OLDEST_RECOVER_STAMP=$TIMESTAMP
		OLDEST_RECOVER=$LOGFILE
	fi
fi
}
###################################################################
# Get list of removable files
GETREMOVABLE() {
if [[ $OLDEST_RESTART < $OLDEST_RECOVER ]] ; then
	OLDEST="S"$OLDEST_RESTART".LOG"
else
	OLDEST="S"$OLDEST_RECOVER".LOG"
fi
SAY "Files newer than and including $OLDEST MUST be retained"

# Check for log wrapping
FIRST=''
for NEXT in `ls $QMLOGS` ; do
	if [ "$FIRST" = '' ] ; then
		FIRST=$NEXT
	fi
	LAST=$NEXT
done
LOGBREAKCOUNT=1
if [ "$FIRST" = 'S0000000.LOG' -a "$LAST" = 'S9999999.LOG' ] ; then
	SAY "MQ Logs are wrapped"
	LOGLIST=''
	TMPLIST=`ls -r $QMLOGS`
	for NEXT in $TMPLIST ; do
		CHKNUM=`echo $NEXT | cut -c2-8`
		if [ "$LASTNUM" = '' ] ; then
			LASTNUM=$CHKNUM
			continue
		fi
		let PLUSNUM=CHKNUM+1
		if [ "$LASTNUM" != "$PLUSNUM" ] ; then
			LEAP='N'
			until [ $LOGBREAKCOUNT -eq $MAXLOGBREAK ] ; do
				let PLUSNUM=PLUSNUM+1
				if [ "$LASTNUM" = "$PLUSNUM" ] ; then
					LEAP='Y'
					LOGBREAKCOUNT=1
					break
				else
					let LOGBREAKCOUNT=LOGBREAKCOUNT+1
				fi
			done
			
			if [ "$LEAP" = 'N' ]; then
				WRAPAT=$LASTNUM
				break
			fi
		fi
		LASTNUM=$CHKNUM
	done
	WRAPAT="S"$WRAPAT".LOG"
	SAY "Logs wrap at $WRAPAT"
	for NEXT in $TMPLIST ; do
		if [[ "$NEXT" < "$WRAPAT" ]] ; then
			LOGLIST="$LOGLIST $NEXT"
		fi
	done
	for NEXT in $TMPLIST ; do
		if [[ "$NEXT" > "$WRAPAT" ]] ; then
			LOGLIST="$LOGLIST $NEXT"
		fi
	done
	LOGLIST="$LOGLIST $WRAPAT"
else
	LOGLIST=`ls -r $QMLOGS`
fi
CANDEL='N'
for NEXT in $LOGLIST ; do
	if [ "$CANDEL" = 'Y' ] ; then
		REMOVABLE_FILES="$REMOVABLE_FILES $NEXT"
	elif [ "$NEXT" = "$OLDEST" ] ; then
		CANDEL='Y'
	fi
done
}
###################################################################
# Begin main program
###################################################################
# Set Variables relating to input parameters
PARMS=$*
QMGR=''
MQROOT='/var/mqm'
LOGGING='N'
LOG_TO=''
ARCHIVE_DIR=''
EXTZIP='N'
VERBOSITY='N'

# Set Default Variables
MAXLOGBREAK=10
MQMESSAGES='AMQ7467 AMQ7468'
OLDEST_RESTART=''
OLDEST_RECOVER=''
OLDEST_RESTART_STAMP='0'
OLDEST_RECOVER_STAMP='0'
REMOVABLE_FILES=''

# Check input parms
if [ $# -eq 0 ] ; then 
	ERR 1
else
# Capture input params
	while [ `echo "$*" | cut -c1` = '-' ] ; do
		case $1 in
			-a) 	shift; ARCHIVE_DIR=$1 ;;
			-d) 	shift; MQROOT=$1 ;;
			-l) 	shift; LOGGING='Y'; LOG_TO=$1 ;;
			-q) 	shift; QMGR=$1 ;;
			-w) 	shift; MAXLOGBREAK=$1 ;;
			-v) 	VERBOSITY='Y' ;;
			-x) 	EXTZIP='Y' ;;
			*) 	: ;;
		esac
		shift
		if [ $# -eq 0 ] ; then break ; fi
	done
fi
# Validate input parms
if [ "$QMGR" = '' ] ; then
	ERR 1
fi	
QMERRORS=$MQROOT/qmgrs/$QMGR/errors
QMLOGS=$MQROOT/log/$QMGR/active
QMINI=$MQROOT/qmgrs/$QMGR/qm.ini
# Validate input directories
if [ ! -d $QMERRORS -o ! -d $QMLOGS ] ; then ERR 2 ; fi
# Validate archive directory
if [ "$ARCHIVE_DIR" = '' ] ; then
	ARCHIVE_DIR=$sQMLOGS
elif [ ! -d $ARCHIVE_DIR ] ; then 
	ERR 3 
fi
# Validate logging file
if [ "$LOGGING" = 'Y' ] ; then
	LOGGING_DIR=`dirname $LOG_TO`
	if [ ! -d $LOGGING_DIR ] ; then ERR 4 ; fi
fi
# Validate Queue Manager qm.ini file
if [ ! -f "$QMINI" ] ; then 
	ERR 5
else
	LOGTYPE=`grep LogType $QMINI | cut -d"=" -f2`
	if [ "$LOGTYPE" != 'LINEAR' ] ; then ERR 6 ; fi
fi
# Validate exclusive options
if [ "$VERBOSITY" = 'Y' -a "$EXTZIP" = 'Y' ] ; then ERR 9 ; fi
###################################################################
SAY "Beginning execution of MQSeries Linear Logfile Maintenance for Queue Manager \"$QMGR\""

# Read in the MQ Logs to determine which files need to be retained
READINLOGS

SAY "Oldest file number necessary for Recovery is \"$OLDEST_RECOVER\""
SAY "Oldest file number necessary for Restart is \"$OLDEST_RESTART\""
GETREMOVABLE
if [ "$REMOVABLE_FILES" = '' ] ; then
	SAY "There are currently no MQ Linear Logfiles available for removal."
else
	SAY "The following files are eligible for removal:\n\t$REMOVABLE_FILES"
	FULLY_QUALIFIED_LIST=''
	for NEXT in $REMOVABLE_FILES ; do
		FULLY_QUALIFIED_LIST="$FULLY_QUALIFIED_LIST $QMLOGS/$NEXT"
	done
	if [ "$EXTZIP" = 'Y' ] ; then
		echo $FULLY_QUALIFIED_LIST
	else
		TIMESTAMP=`date +"%Y%m%d-%H%M%S"`
		TARFILE="$ARCHIVE_DIR/MQLINEARLOGS_$TIMESTAMP.tar"
		
		# Perform the native UNIX archive and cleanup steps
		tar -cvf $TARFILE $FULLY_QUALIFIED_LIST
		compress $TARFILE
		rm $FULLY_QUALIFIED_LIST
	fi
fi
###################################################################
SAY "Execution of MQSeries Linear Logfile Maintenance for Queue Manager \"$QMGR\" Completed."
exit $?